/* wptOwnertrust.cpp - Ownertrust im- and export
 *	Copyright (C) 2001, 2002, 2003, 2005 Timo Schulz
 *	Copyright (C) 2005 g10 Code GmbH
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <sys/stat.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptGPG.h"
#include "wptErrors.h"
#include "wptNLS.h"
#include "wptW32API.h"


/* Export the ownertrust to the file @fname.
   Return value: 0 on success. */
static gpgme_error_t
gpg_export_ownertrust (const char *fname)
{
    gpgme_error_t err;
    char *ot_data = NULL;
    FILE *f;

    err = gpg_manage_ownertrust (&ot_data, 1);
    if (err)
	return err;
    f = fopen (fname, "wb");
    if (f) {
	fwrite (ot_data, 1, strlen (ot_data), f);
	fclose (f);
    }
    else
	err = gpg_error (GPG_ERR_ENOENT);
    free (ot_data);
    return 0;
}


/* Import the ownertrust from file @fname.
   Return value: 0 on success. */
static gpgme_error_t
gpg_import_ownertrust (const char *fname)
{
    gpgme_error_t err;
    FILE *f;
    struct stat st;
    char *ot_data;

    if (stat (fname, &st))
	return gpg_error (GPG_ERR_ENOENT);
    f = fopen (fname, "rb");
    if (!f)
	return gpg_error (GPG_ERR_ENOENT);
    ot_data = (char*)calloc (1, st.st_size+1);
    if (!ot_data)
	BUG (NULL);
    fread (ot_data, 1, st.st_size, f);
    fclose (f);

    err = gpg_manage_ownertrust (&ot_data, 0);

    free (ot_data);
    return 0;
}


/* Dialog procedure to manage the ownertrust. */
BOOL CALLBACK
ownertrust_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    gpgme_error_t err;
    const char *fname;
    
    switch( msg ) {
    case WM_INITDIALOG:
    #ifndef LANG_DE
        SetWindowText( dlg, _("Ownertrust") );
        SetDlgItemText( dlg, IDC_OWNERTRUST_INFO, _("File") );
        SetDlgItemText( dlg, IDC_OWNERTRUST_DESC,
                       _("Here it is possible to save or restore the ownertrust from the "
                         "'trustdb' file. This could be very useful because the values are NOT "
                         "stored in the keyring.") );
    #endif
	center_window( dlg, NULL );
        SetForegroundWindow( dlg );
        return TRUE;
        
    case WM_SYSCOMMAND:
        if (LOWORD (wparam) == SC_CLOSE)
            EndDialog (dlg, TRUE);
        return TRUE;
        
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDC_OWNERTRUST_EXPORT:
	    fname = get_filesave_dlg (dlg, _("Select file name for output"), NULL, NULL);
	    if (!fname)
		return TRUE;
	    err = gpg_export_ownertrust (fname);
            if (err) {
                msg_box (dlg, gpgme_strerror( err ), _("Ownertrust"), MB_ERR );
                return TRUE;
            }
            status_box (dlg, _("Ownertrust successfully exported."), _("GnuPG Status"));
            return TRUE;

	case IDC_OWNERTRUST_IMPORT:
	    fname = get_fileopen_dlg (dlg, _("Select file name for input"), NULL, NULL);
	    if (!fname)
		return TRUE;
	    err = gpg_import_ownertrust (fname);
	    if (err) {
		msg_box (dlg, gpgme_strerror (err), _("Ownertrust"), MB_ERR);
		return TRUE;
	    }
	    status_box (dlg, _("Ownertrust succefully imported."), _("GnuPG Status"));
	    return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return TRUE;
        }
        break;
    }
    
    return FALSE;
}
